/* bis_interface.c - A BIS interface to MX. */

#include <stdio.h>
#include <stdlib.h>

#include "mx_record.h"
#include "mx_motor.h"

int
main( int argc, char *argv[] )
{
	MX_RECORD *record_list, *motor_record;
	char buffer[200];
	char motor_name[ MXU_RECORD_NAME_LENGTH + 1 ];
	size_t length;
	double position, destination;
	int num_items;
	mx_status_type mx_status;

	setvbuf(stdin, (char *) NULL, _IOLBF, 0);

	if ( argc != 2 ){
		fprintf(stderr,"Usage: %s database_name\n", argv[0]);
		exit(0);
	}

	mx_status = mx_setup_database( &record_list, argv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	while( 1 ) {
		fgets( buffer, sizeof buffer, stdin );

		if ( feof(stdin) || ferror(stdin) )
			break;		/* Exit the while() loop. */

		/* Zap the newline at the end of the buffer. */

		length = strlen(buffer);

		if ( buffer[length-1] == '\n' ) {
			buffer[length-1] = '\0';
		}

		/* Execute the command. */

		if ( strncmp( buffer, "move", 4 ) == 0 ) {
			num_items = sscanf( buffer, "%*s %s %lg",
						motor_name, &destination );

			if ( num_items != 2 ) {
				fprintf( stderr,
				"Error: Cannot parse move command '%s'\n",
					buffer );

				/* Go back to the top of the while loop. */

				continue;
			}

			motor_record = mx_get_record( record_list, motor_name );

			if ( motor_record == NULL ) {
				fprintf(stderr,
			"Error: The motor record '%s' does not exist.\n",
					motor_name );

				/* Go back to the top of the while loop. */

				continue;
			}

			mx_status = mx_motor_move_absolute( motor_record,
							destination, 0 );

			if ( mx_status.code != MXE_SUCCESS ) {

				/* Go back to the top of the while loop. */

				continue;
			}

			mx_status = mx_wait_for_motor_stop( motor_record, 0 );

			if ( mx_status.code == MXE_SUCCESS ) {
				fprintf(stderr, "Move complete.\n" );
			}
		} else if ( strncmp( buffer, "get_position", 12 ) == 0 ) {
			num_items = sscanf( buffer, "%*s %s", motor_name );

			if ( num_items != 1 ) {
				fprintf( stderr,
			"Error: Cannot parse get_position command '%s'\n",
					buffer );

				/* Go back to the top of the while loop. */

				continue;
			}

			motor_record = mx_get_record( record_list, motor_name );

			if ( motor_record == NULL ) {
				fprintf(stderr,
			"Error: The motor record '%s' does not exist.\n",
					motor_name );

				/* Go back to the top of the while loop. */

				continue;
			}

			mx_status = mx_motor_get_position( motor_record,
								&position );

			if ( mx_status.code == MXE_SUCCESS ) {
				fprintf( stderr,"%g\n", position );
			}
		} else {
			fprintf(stderr, "Error: Unrecognized command '%s'\n",
					buffer );
		}
	}

	fprintf(stderr, "Error: Connection with client lost, shutting down.\n");

	exit(0);
}

